/**************************************************************************/
/*  openxr_fb_passthrough_geometry.h                                      */
/**************************************************************************/
/*                       This file is part of:                            */
/*                              GODOT XR                                  */
/*                      https://godotengine.org                           */
/**************************************************************************/
/* Copyright (c) 2022-present Godot XR contributors (see CONTRIBUTORS.md) */
/*                                                                        */
/* Permission is hereby granted, free of charge, to any person obtaining  */
/* a copy of this software and associated documentation files (the        */
/* "Software"), to deal in the Software without restriction, including    */
/* without limitation the rights to use, copy, modify, merge, publish,    */
/* distribute, sublicense, and/or sell copies of the Software, and to     */
/* permit persons to whom the Software is furnished to do so, subject to  */
/* the following conditions:                                              */
/*                                                                        */
/* The above copyright notice and this permission notice shall be         */
/* included in all copies or substantial portions of the Software.        */
/*                                                                        */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
/**************************************************************************/

#ifndef OPENXR_FB_PASSTHROUGH_GEOMETRY_H
#define OPENXR_FB_PASSTHROUGH_GEOMETRY_H

#include <godot_cpp/classes/mesh.hpp>
#include <godot_cpp/classes/mesh_instance3d.hpp>
#include <godot_cpp/classes/node3d.hpp>

#include <openxr/openxr.h>

namespace godot {
class OpenXRFbPassthroughGeometry : public Node3D {
	GDCLASS(OpenXRFbPassthroughGeometry, Node3D)

private:
	OpenXRFbPassthroughGeometry();

	void create_passthrough_geometry();
	void destroy_passthrough_geometry();
	void update_passthrough_geometry_transform();

	void instatiate_opaque_mesh();
	void delete_opaque_mesh();

	Ref<Mesh> mesh;
	bool enable_hole_punch = true;
	XrGeometryInstanceFB geometry_instance = XR_NULL_HANDLE;
	MeshInstance3D *opaque_mesh = nullptr;

protected:
	void _notification(int p_what);

	static void _bind_methods();

public:
	void set_mesh(const Ref<Mesh> &p_mesh);
	Ref<Mesh> get_mesh() const;

	void set_enable_hole_punch(bool p_enable);
	bool get_enable_hole_punch() const;
};
} //namespace godot

#endif
